const { ccclass, property } = cc._decorator;

/** 
 * 花洒功能 
 * author: Howe.Ma & 论坛广大网友
 */
@ccclass
export default class Shower extends cc.Component {

    @property(cc.Node)
    game_bg: cc.Node = null;
    @property(cc.Prefab)
    linePer: cc.Prefab = null; //绳子预制体
    @property(cc.Integer)
    softDegree: number = 0; //绳子总的小节数，越多则绳子越柔软
    @property(cc.Node)
    anchor: cc.Node = null; //静态刚体锚点node

    @property(cc.Node)
    LastNOde: cc.Node = null; //最后node

    //水滴层
    @property(cc.Node)
    private NewWaterLoad: cc.Node = null;

    //手指滑动速度值
    // private MoveScale: cc.Vec2 = new cc.Vec2(20, 20);
    //手指是否点击
    private IsClick: boolean = false;
    // private touchStartPosition: cc.Vec2 = cc.v2();

    //创建间隔
    // private CreateTime: number = 0;

    @property(cc.Node)
    DragBoundary: cc.Node = null;

    tTwe: cc.Tween = null;
    // 旋转速度默认180即可
    _rotationSpeed = 180;

    ropeCutArray = new Array();

    /** 是否可以喷水了 */
    _canTouchShower: boolean = true;

    scheduleFunc: Function = null;   // 时间调度器的回调方法
    touchedTime = 0; // 触摸的时间 /s

    // 处理快速下滑及 超过范围内夹住的需求
    boundsPosition;
    boundsSize;
    movementBounds: cc.Node;
    outOfBounds = false;
    offset = cc.v2(0, 0);

    onLoad() {
 
        let size = cc.view.getFrameSize();
        // ipad竖屏
        if (size.width < size.height) {
            // cc.log('竖屏')
        } else {    // ipad横屏
            // cc.log('横屏')
        }

        cc.view.setResizeCallback((params?) => {
            let size = cc.view.getFrameSize();
            // ipad竖屏
            if (size.width < size.height) {
                cc.log('竖屏')
            } else {    // ipad横屏
                cc.log('横屏')
            }
            if (cc.isValid(this.node)) {
                this.node.active = false;
            }
            setTimeout(() => {
                if (cc.isValid(this.node) && this.node.active === false) {
                    try {
                        this.node.active = true;
                    } catch (error) {
                        this.LastNOde.getChildByName('image-center').active = false;
                        this.scheduleOnce(() => {
                            this.LastNOde.getChildByName('image-center').active = true;
                        })
                    }
                }
            }, 500)

        })

        cc.director.getPhysicsManager().enabled = true; //开启物理引擎
        // cc.director.getPhysicsManager().debugDrawFlags = cc.PhysicsManager.DrawBits.e_aabbBit |
        // cc.PhysicsManager.DrawBits.e_jointBit |
        // cc.PhysicsManager.DrawBits.e_shapeBit
        // ;

        cc.director.getPhysicsManager().gravity = cc.v2(0, 0); // 更改重力值
        this.game_bg.opacity = 0;

        // 记录触摸点与拖拽节点的偏移量--放到onload里
        this.movementBounds = this.DragBoundary;
        this.offset = cc.v2(0, 0);
        this.boundsPosition = this.movementBounds.position;
        this.boundsSize = this.movementBounds.getContentSize();
    }


    protected onEnable(): void {
        cc.director.getPhysicsManager().enabled = true; //开启物理引擎
        /**
 * 在传统的JavaScript中，函数的作用域是动态绑定的，也就是说在函数被调用时，它的this值会根据调用方式来确定。
    在事件监听器中，函数作为回调函数被传递给事件处理器，当事件触发时，函数会被调用，此时函数内部的this值指向触发事件的对象。
    然而，对于普通函数而言，它们的this值是在运行时绑定的，而不是在函数定义时绑定的。
    这就导致在事件监听器中使用普通函数作为回调时，函数内部的this值可能会发生改变，不再指向预期的对象。
    为了避免这个问题，可以使用箭头函数语法。箭头函数不会创建自己的this值，而是会继承外部作用域的this值。
    这意味着在箭头函数内部，this值与外部作用域中的this值是一样的，不会发生改变。
    在你提供的代码示例中，通过使用箭头函数语法来替代事件监听器中的普通函数，确保了函数内部的this值与外部作用域的this值保持一致，
    从而避免了绑定函数时可能出现的问题。
 *
 */
        this.LastNOde.on(cc.Node.EventType.TOUCH_START, (event) => this._touchStart(event));
        this.LastNOde.on(cc.Node.EventType.TOUCH_MOVE, (event) => this._touchMove(event));
        this.LastNOde.on(cc.Node.EventType.TOUCH_END, (event) => this._touchEnd(event));
        this.LastNOde.on(cc.Node.EventType.TOUCH_CANCEL, (event) => this._touchEnd(event));
        //添加鼠标移动 - 测试
        // let joint = this.LastNOde.addComponent(cc.MouseJoint);
        // joint.mouseRegion = this.LastNOde;
        // joint.apply();


        this.onInit();
    }

    protected onDisable(): void {
        cc.director.getPhysicsManager().enabled = false; //关闭物理引擎
        this.LastNOde.off(cc.Node.EventType.TOUCH_START, (event) => this._touchStart(event));
        this.LastNOde.off(cc.Node.EventType.TOUCH_MOVE, (event) => this._touchMove(event));
        this.LastNOde.off(cc.Node.EventType.TOUCH_END, (event) => this._touchEnd(event));
        this.LastNOde.off(cc.Node.EventType.TOUCH_CANCEL, (event) => this._touchEnd(event));
    }


    /**
     * 触摸计时
     */
    onTouchSchedule() {
        // 计时间隔
        const scheduleInterval = cc.director.getDeltaTime();
        // this.touchedTime
        if (this.scheduleFunc) {
            this.unschedule(this.scheduleFunc);
        }
        this.touchedTime = 0;
        // 计时方法
        this.scheduleFunc = () => {
            if (this.IsClick) {
                this.touchedTime += scheduleInterval;
                // cc.log('触摸计时 ', this.touchedTime);
                if (this.touchedTime > 0.1) {
                    if (this._canTouchShower) {
                        this.NewWaterLoad.active = true;

                        this.unschedule(this.scheduleFunc);
                    }
                }
            }
        };
        // 开始计时
        this.schedule(this.scheduleFunc, scheduleInterval);
    }

    private _touchStart(event: cc.Event.EventTouch) {
        // cc.log('_touchStart this.LastNOde.angle ', this.LastNOde.angle)
        this.onChangeMaxLength(true);

        this.IsClick = true;
        cc.director.getPhysicsManager().gravity = cc.v2(0, -500); // 更改重力值
        this.LastNOde.getComponent(cc.RigidBody).type = cc.RigidBodyType.Animated;

        // this.touchStartPosition = event.getLocation();
        if (this._canTouchShower) {
            this.onTouchSchedule();
            //this.NewWaterLoad.active = true;
        }

        // 获取偏移量
        const touchPos = event.getLocation();
        const nodePos = this.LastNOde.parent.convertToNodeSpaceAR(touchPos);
        this.offset = cc.v2(nodePos.x - this.LastNOde.x, nodePos.y - this.LastNOde.y);
        // this.anchor.setPosition(0,this.anchor.y - 50);

    }


    private _touchMove(event: cc.Event.EventTouch) {
        if (!this.IsClick) return;
        // 方案一
        // const touchPos = event.getLocation();
        // const nodePos = this.LastNOde.parent.convertToNodeSpaceAR(touchPos);

        // const minX = this.boundsPosition.x - this.boundsSize.width / 2;
        // const maxX = this.boundsPosition.x + this.boundsSize.width / 2;
        // const minY = this.boundsPosition.y - this.boundsSize.height / 2;
        // const maxY = this.boundsPosition.y + this.boundsSize.height / 2;

        // let x = nodePos.x - this.offset.x;
        // let y = nodePos.y - this.offset.y;

        // // 手动夹紧位置
        // if (x < minX) {
        //     x = minX;
        // } else if (x > maxX) {
        //     x = maxX;
        // }

        // if (y < minY) {
        //     y = minY;
        // } else if (y > maxY) {
        //     y = maxY;
        // }

        this.updateRoge();


        let delta = event.getDelta(); //获取触点距离上一次事件移动的距离对象，对象包含 x 和 y 属性
        const touchPos = event.getLocation();
        const nodePos = this.LastNOde.parent.convertToNodeSpaceAR(touchPos);

        // 计算当前节点的旋转角度
        // 计算目标旋转角度
        let targetAngle = 360 - this.getAngle(this.LastNOde.x, this.LastNOde.y, delta.x, delta.y);
        // 计算旋转差值
        let deltaAngle = targetAngle - this.LastNOde.angle;;
        // 将旋转差值调整到 -180 到 180 的范围内
        deltaAngle = ((deltaAngle + 180) % 360) - 180;
        // 设置一个旋转速度
        // 根据旋转速度和时间间隔计算本次更新的旋转角度
        let rotateAmount = this._rotationSpeed * cc.director.getDeltaTime();
        // 根据旋转差值的正负来确定旋转方向
        if (deltaAngle > 0) {
            rotateAmount = Math.ceil(Math.min(rotateAmount, deltaAngle));
        } else {
            rotateAmount = Math.ceil(Math.max(-rotateAmount, deltaAngle));
        }
        // 更新节点的旋转角度
        this.LastNOde.angle = Math.ceil(this.LastNOde.angle += rotateAmount);
        // this.LastNOde.setPosition(x, y);



        const minX = this.boundsPosition.x - this.boundsSize.width * this.movementBounds.anchorX;
        const maxX = this.boundsPosition.x + this.boundsSize.width * (1 - this.movementBounds.anchorX);
        const minY = this.boundsPosition.y - this.boundsSize.height * this.movementBounds.anchorY;
        const maxY = this.boundsPosition.y + this.boundsSize.height * (1 - this.movementBounds.anchorY);

        let x = nodePos.x - this.offset.x;
        let y = nodePos.y - this.offset.y;

        // 手动夹紧位置
        x = Math.min(Math.max(x, minX), maxX);
        y = Math.min(Math.max(y, minY), maxY);

        // 计算当前位置和上一帧位置的距离
        const delta2 = cc.v2(x - this.LastNOde.x, y - this.LastNOde.y);
        const distance = delta2.mag();
        // 设置速度限制阈值
        const maxSpeed = 18; // 根据需求设置合适的值
        if (distance > maxSpeed) {
            // cc.log('移动时的向量-', distance)
            // 如果超过速度限制，则将距离缩小到限制范围内
            delta2.mulSelf(maxSpeed / distance);
            // cc.log('缩放后当前向量-', delta2)

        }

        // 更新位置
        this.LastNOde.x += delta2.x;
        this.LastNOde.y += delta2.y;


        // // this.updateRoge()
        // //声明矩形区域
        // let rec1: cc.Rect = this.LastNOde.getChildByName('image-center').getBoundingBoxToWorld();//声明矩形区域
        // let rec2: cc.Rect = this.DragBoundary.getBoundingBoxToWorld();
        // let intersecRec = new cc.Rect();
        // rec1.intersection(intersecRec, rec2);//获得相交区域
        // let isIntersect: boolean = rec1.intersects(rec2);//判断是否相交
        // if (!isIntersect) {
        //     cc.log('不相交了')
        //     this._touchEnd(event);
        //     // this.IsClick = false;
        //     return;
        // }
        // // cc.log('花洒的位置', this.LastNOde.position.x, this.LastNOde.position.y);


        // //向量移动
        // // let updatedDelta = this.rotateVector(delta, cc.misc.degreesToRadians(rotateAmount));
        // //和上面是一样的
        // this.LastNOde.x += delta.x;
        // this.LastNOde.y += delta.y;
        // // this.LastNOde.position = this.LastNOde.position.add(cc.v3(updatedDelta));

        // // 测试调用限定最大距离的方法
        // // this.limitMaxDistance(true);

    }

    private _touchEnd(event: cc.Event.EventTouch) {
        this.IsClick = false;
        // this.anchor.setPosition(0,this.anchor.y + 50);

        // 重置偏移量
        this.offset = cc.v2(0, 0);
        this.onChangeMaxLength(false);
        cc.director.getPhysicsManager().gravity = cc.v2(0, -300); // 更改重力值
        this.LastNOde.getComponent(cc.RigidBody).linearVelocity = cc.v2();
        this.LastNOde.getComponent(cc.RigidBody).type = cc.RigidBodyType.Dynamic;
        // cc.log(this.LastNOde.angle, ' _touchEnd时的角度')

        if (this.tTwe) {
            this.tTwe.stop();
            this.LastNOde.getComponent(cc.RigidBody).type = cc.RigidBodyType.Dynamic;
        }
        this.tTwe = cc.tween(this.LastNOde).to(0.3, { angle: 0 }).call(() => { this.updateRoge(true) }).start();
        this.NewWaterLoad.active = false;
        this.touchedTime = 0;
        this.unschedule(this.scheduleFunc);
    }

    onChangeMaxLength(status: boolean) {
        return
        let maxLength = 12;
        // 获取多个Rope Joint的组件
        let ropeJoints = this.ropeCutArray;//this.node.getComponents(cc.RopeJoint);
        let lastLeng = ropeJoints[this.ropeCutArray.length - 1].getComponent(cc.RopeJoint).maxLength;

        cc.log(this.ropeCutArray[3].getComponent(cc.RopeJoint).maxLength, '当前的maxlength')
        if (status) {
            maxLength = 12;// this.ropeCutArray[3].getComponent(cc.RopeJoint).maxLength * 3
            // ropeCutArray[i].getComponent(cc.RopeJoint).maxLength = ropeCutArray[i].height * 0.1;
            ropeJoints[this.ropeCutArray.length - 1].getComponent(cc.RopeJoint).maxLength = 26 + maxLength;
            ropeJoints[this.ropeCutArray.length - 1].getComponent(cc.RopeJoint).apply();


        } else {
            ropeJoints[this.ropeCutArray.length - 1].getComponent(cc.RopeJoint).maxLength = 26;
            ropeJoints[this.ropeCutArray.length - 1].getComponent(cc.RopeJoint).apply();

            maxLength = 5;// this.ropeCutArray[3].getComponent(cc.RopeJoint).maxLength * 0.5;
            // return
        }

        // // 定义过渡时间和帧数
        // let transitionTime = 0.1; // 过渡时间为0.5秒
        // let frames = 60; // 假设使用60帧来完成过渡


        //         // 计算每帧需要变化的长度
        //         let distanceDelta = (5 - ropeJoint.maxLength) / frames;

        //         // 使用定时器或动画系统逐帧改变最大距离，实现过渡效果
        //         let frameCount = 0;
        //         let timer = setInterval(() => {
        //             frameCount++;
        //             ropeJoint.maxLength += distanceDelta;

        //             if (frameCount >= frames) {
        //                 clearInterval(timer);
        //                 ropeJoint.maxLength = 5;
        //             }

        //             ropeJoint.apply();
        //         }, transitionTime / frames * 1000);


        // 遍历每个Rope Joint
        for (let i = 0; i < ropeJoints.length - 1; i++) {
            let ropeJoint = ropeJoints[i].getComponent(cc.RopeJoint);
            if (ropeJoint) {

                // 获取当前距离
                let currentDistance = cc.Vec2.distance(ropeJoint.connectedBody.node.getPosition(), ropeJoint.node.getPosition());

                // 如果当前距离超过最大距离，则修改约束长度为最大距离，并调用apply()方法使其生效
                ropeJoint.maxLength = maxLength;// currentDistance;
                ropeJoint.apply();
            }
        }

    }


    limitMaxDistance(isover) {
        // 获取多个Rope Joint的组件
        let ropeJoints = this.ropeCutArray;//this.node.getComponents(cc.RopeJoint);

        // 遍历每个Rope Joint
        for (let i = 0; i < ropeJoints.length; i++) {
            let ropeJoint = ropeJoints[i].getComponent(cc.RopeJoint);
            // 获取当前距离
            let currentDistance = cc.Vec2.distance(ropeJoint.connectedBody.node.getPosition(), ropeJoint.node.getPosition());

            // 如果当前距离超过最大距离，则修改约束长度为最大距离，并调用apply()方法使其生效
            if (currentDistance > ropeJoint.maxLength) {
                if (isover) ropeJoint.maxLength = 10;// currentDistance;
                // cc.log(currentDistance, '最大距离')

                ropeJoint.apply();
            }
        }

        // 遍历每个Rope Joint
        //     for (let i = 0; i < ropeJoints.length; i++) {
        //       let ropeJoint = ropeJoints[i];

        //       // 获取当前距离
        //       let currentDistance = cc.Vec2.distance(ropeJoint.connectedBody.worldPosition, ropeJoint.node.worldPosition);

        //       // 如果当前距离超过最大距离，则修改约束长度为最大距离，并调用apply()方法使其生效
        //       if (currentDistance > ropeJoint.maxLength) {
        //         ropeJoint.maxLength = currentDistance;
        //         cc.log(currentDistance,'最大距离')
        //         ropeJoint.apply();
        //       }
        //     }
    }

    fomatFloat(src, pos) {
        return Math.round(src * Math.pow(10, pos)) / Math.pow(10, pos);
    }

    onInit() {
        let slef = this;
        cc.director.getPhysicsManager().gravity = cc.v2(0, -300); // 更改重力值
        this.LastNOde.getComponent(cc.RigidBody).type = cc.RigidBodyType.Dynamic;
        // this.anchor.setPosition(0,this.anchor.y + 50);

        var ropeCutArray = new Array(); //存放所有绳子小节的数组
        // let fatherNode= cc.find('Canvas')

        //遍历这个数组
        for (var i = 0; i < this.softDegree; i++) {
            ropeCutArray[i] = cc.instantiate(this.linePer); //生成新的绳子小节存放到数组中
            this.node.addChild(ropeCutArray[i]); //将绳子小节添加为Canvas的子节点，这里的this.node指的是Canvas
            // fatherNode.addChild(ropeCutArray[i]); //将绳子小节添加为Canvas的子节点，这里的this.node指的是Canvas
            //如果是第一个绳子小节，那么它的Connected Body就应该是anchor
            if (i == 0) {
                ropeCutArray[i].getComponent(cc.RopeJoint).connectedBody = this.anchor.getComponent(cc.RigidBody);
                //修改绳子的最大长度
                ropeCutArray[i].getComponent(cc.RopeJoint).maxLength = this.fomatFloat(ropeCutArray[i].height * 0.15, 2);//this.anchor.width * 0.1;
                // ropeCutArray[i].getComponent(cc.RopeJoint).maxLength = (this.anchor.width + ropeCutArray[i].width) / 2;
            }
            //如果是第2~n个绳子小节，那么它的Connected Body就应该是上一个绳子小节，直接数组索引-1就可以得到
            else {
                ropeCutArray[i].getComponent(cc.RopeJoint).connectedBody = ropeCutArray[i - 1].getComponent(cc.RigidBody);
                //修改绳子的最大长度
                ropeCutArray[i].getComponent(cc.RopeJoint).maxLength = this.fomatFloat(ropeCutArray[i].height * 0.1 + (i * 0.3), 2) + 1;
                // if(i <  this.softDegree * 0.5){
                // }else{
                //     ropeCutArray[i].getComponent(cc.RopeJoint).maxLength = this.fomatFloat(ropeCutArray[i].height * 0.1 + (i * 0.3), 2) + 1;
                // }
                //cc.log('每个关节的最大距离',i, ropeCutArray[i].getComponent(cc.RopeJoint).maxLength );

            }

            //修改完RopeJoint的属性之后一定要调用apply()方法，不然所有修改都无效！
            ropeCutArray[i].getComponent(cc.RopeJoint).apply();
        }

        let maxAll = 0;
        // 修正这个数组最大长度
        for (let i = 0; i < ropeCutArray.length; i++) {
            const element = ropeCutArray[i].getComponent(cc.RopeJoint);
            // if(i < ropeCutArray.length * 0.5 ){ // 反
            //     element.maxLength = this.fomatFloat(ropeCutArray[i].height * 0.1 + ((ropeCutArray.length - i) * 0.3), 2) + 1;
            // }else{
            //     element.maxLength = this.fomatFloat(ropeCutArray[i].height * 0.1 + ((ropeCutArray.length - i) * 1 ), 2) + 1;
            // }
            // 正
            ropeCutArray[i].getComponent(cc.RopeJoint).maxLength = this.fomatFloat(ropeCutArray[i].height * 0.1 + (i * 0.3), 2);
            // cc.log('element每个关节的最大距离', i, element.maxLength);
            element.apply();
            maxAll += element.maxLength;
        }
        cc.log(maxAll, '-总')

        ropeCutArray.push(this.LastNOde);
        //this.node.addChild(this.LastNOde);
        ropeCutArray[ropeCutArray.length - 1].getComponent(cc.RopeJoint).connectedBody = ropeCutArray[ropeCutArray.length - 2].getComponent(cc.RigidBody);
        //修改绳子的最大长度
        ropeCutArray[ropeCutArray.length - 1].getComponent(cc.RopeJoint).maxLength = this.LastNOde.height * 0.15;//ropeCutArray[ropeCutArray.length - 2].width * 0.1;
        ropeCutArray[ropeCutArray.length - 1].getComponent(cc.RopeJoint).apply();
        this.LastNOde.setSiblingIndex(cc.macro.MAX_ZINDEX)
        this.game_bg.opacity = 255;
        // 花洒入场音效
        // AudioManager.getInstance().play_sound('operations/24', true, false);

        for (let index = 0; index < ropeCutArray.length; index++) {
            this.ropeCutArray.push(ropeCutArray[index])
        }
        // this._touchEnd(null);

    }

    // 验证绳子是否超出最大距离
    updateRoge(isNeed = false) {
        var rArray = this.ropeCutArray; //存放所有绳子小节的数组
        let canMove = true;
        //遍历这个数组
        for (var i = this.ropeCutArray.length - 2; i > 0; i--) {
            // ropeCutArray[i].setPosition(cc.v2(0,0))
            //如果是第2~n个绳子小节，那么它的Connected Body就应该是上一个绳子小节，直接数组索引-1就可以得到
            if (i != 0) {
                const distance = this.ropeCutArray[i].position.sub(this.ropeCutArray[i + 1].position).mag();
                if (rArray[i + 1] != this.LastNOde) {
                    if (distance > this.ropeCutArray[this.ropeCutArray.length - 1].height * 0.2 + 8) { //+ 20
                        // this.ropeCutArray[i].getComponent(cc.RopeJoint).maxLength = this.anchor.width * 0.5;

                        // cc.log('超出最大距离,两个节点之间的距离', distance, '  index是 ', i + 1);
                        // this.ropeCutArray[i].getComponent(cc.RopeJoint).apply();
                        this._touchEnd(null);
                        // this.IsClick = false;
                        canMove = false;
                        break;
                    }
                }
            }
            //如果是第一个绳子小节，那么它的Connected Body就应该是anchor
        }

        if (isNeed) {
            // cc.log('updateRoge 准备修正角度', this.LastNOde.angle)
            this.LastNOde.getComponent(cc.RigidBody).type = cc.RigidBodyType.Animated;
            this.LastNOde.angle = 0;
            this.LastNOde.getComponent(cc.RigidBody).type = cc.RigidBodyType.Dynamic;
        }
        return canMove;
    }

    /**
     * 获取两点间距离
     * @param p1X
     * @param p1Y
     * @param p2X
     * @param p2Y
     * @returns {number}
     */
    public getDistance(p1X: number, p1Y: number, p2X: number, p2Y: number): number {
        var disX: number = p2X - p1X;
        var disY: number = p2Y - p1Y;
        var disQ: number = disX * disX + disY * disY;
        return Math.sqrt(disQ);
    }


    /**
     * 获得两点之间角度
     * @param _firstpoint
     * @param _secondpoint
     */
    private getPointAngle(_firstpoint: cc.Vec2, _secondpoint: cc.Vec2) {
        return Math.atan2(_secondpoint.y - _firstpoint.y, _secondpoint.x - _firstpoint.x);
    }

    // 旋转矢量
    rotateVector(vector: cc.Vec2, angle: number): cc.Vec2 {
        let cosVal = Math.cos(angle);
        let sinVal = Math.sin(angle);
        let x = vector.x * cosVal - vector.y * sinVal;
        let y = vector.x * sinVal + vector.y * cosVal;
        return cc.v2(x, y);
    }


    // https://blog.csdn.net/programmer_trip/article/details/102818436
    // 获取角度-朝向 //获得人物中心和鼠标坐标连线，与y轴正半轴之间的夹角
    getAngle(px, py, mx, my) {
        var x = Math.abs(px - mx);
        var y = Math.abs(py - my);
        var z = Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
        var cos = y / z;
        var radina = Math.acos(cos);//用反三角函数求弧度
        var angle = Math.floor(180 / (Math.PI / radina));//将弧度转换成角度--//将弧度转换为欧拉角
        if (mx > px && my > py) {//鼠标在第四象限
            angle = 180 - angle;
        }

        if (mx == px && my > py) {//鼠标在y轴负方向上
            angle = 180;
        }

        if (mx > px && my == py) {//鼠标在x轴正方向上
            angle = 90;
        }

        if (mx < px && my > py) {//鼠标在第三象限
            angle = 180 + angle;
        }

        if (mx < px && my == py) {//鼠标在x轴负方向
            angle = 270;
        }

        if (mx < px && my < py) {//鼠标在第二象限
            angle = 360 - angle;
        }
        return angle;
    }

    protected onDestroy(): void {
    }

}
